library(lme4)

mkSampler <- function(a=-0.5,sigX=1.1, b=0.5,OVP=1,IOVP=-1,sigY=1, sigSub=1) {
  function (Nsub=30, Nsam=100) {
    u <-  runif(Nsub*Nsam, -1, 1)
    v <-  runif(Nsub*Nsam, -1, 1)
    ex <- sigX*rnorm(Nsub*Nsam)   
    ey <- sigY*rnorm(Nsub*Nsam)
    xsub <- sigSub*rnorm(Nsub)
    ysub <- sigSub*rnorm(Nsub)
    sub  <- as.factor(do.call(c, args=lapply(seq(1,Nsub,1), function (i) { rep(i, Nsam) })))
    Xm <-  a*u + xsub[sub]
    X  <- Xm + ex
    Y <-  b*v + OVP*Xm**2 + IOVP*ex**2 + ysub[sub] + ey
    data.frame(X=X, Y=Y, u=u, v=v, sub=sub)
  }
}

mkJoinedDevFun <- function(data) {
  X2 <- data$X
  modX <- lmer(X2 ~ 0 + (1|sub), data=data, REML=FALSE)
  Y2 <- data$Y
  modY <- lmer(Y2 ~ v + (1|sub), data=data, REML=FALSE)
  devfun <- function (theta) {
    X2 <- data$X - data$u*theta[1]
    modX <- refit(modX, X2)
    # predict mean and resid
    Xm <- data$u*theta[1]+predict(modX); Xr <- residuals(modX)
    Y2 <- data$Y - poly(Xm,2,raw=T)%*%theta[2:3] - poly(Xr,2,raw=T)%*%theta[4:5]
    # fit second model
    modY <- refit(modY, Y2)
    # return sum of deviances
    deviance(modX) + deviance(modY)
  }    
  init <- c(0, 0, 0, 0, 0)
  lower <- c(-Inf, -Inf, -Inf, -Inf, -Inf)
  list(init, fn=devfun, lower=lower, method="L-BFGS-B")
}

sampler <- mkSampler()

simstep <- function() {
  data <- sampler()
  # separate fits
  modX0 <- lmer(X ~ u + (1|sub), data=data, REML=FALSE)
  modY0 <- lmer(Y ~ v + poly(X,2,raw=T) + (1|sub), data=data, REML=FALSE)
  # sequential fit of linked models
  Xm <- predict(modX0); Xr <- residuals(modX0)
  modY1 <- lmer(Y ~ v + poly(Xm,2,raw=T) + poly(Xr,2,raw=T) + (1|sub), data=data, REML=FALSE)
  devfun <- mkJoinedDevFun(data)
  opt_res <- do.call(optim, devfun)
  # a0, a1, b1, a2, b2
  c(fixef(modY0)[4], fixef(modY1)[4], fixef(modY1)[6], opt_res$par[3], opt_res$par[5])
}

Nstep = 1000
res <- replicate(Nstep, simstep())
rownames(res)<- c("X^2", "sXm^2", "sXr^2", "jXm^2", "jXr^2")
saveRDS(res, "nonlinSim_result.rds")

